/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_CONTROLWAYPOINTSTRUCT_H
#define DPC_IMPL_TYPE_CONTROLWAYPOINTSTRUCT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_uint32.h"
#include "dpc/impl_type_vector_uint8_t.h"
#include "impl_type_float.h"
#include "dpc/impl_type_vector_wpextendlanestruct.h"

namespace dpc {
struct ControlWayPointStruct {
    ::UInt8 control_waypoint_target_road_type{ 0U };
    ::UInt32 control_waypoint_id{ 0U };
    ::dpc::vector_uint8_t control_waypoint_lane_id_list;
    ::dpc::vector_uint8_t control_waypoint_raw_lane_id_list;
    ::Float control_waypoint_offset{ 0.0F };
    ::dpc::vector_WPExtendLaneStruct control_wp_extendlane_list;
    ::UInt8 deceleration_lane_type{ 0U };
    ::UInt8 control_waypoint_speed_limit{ 0U };

    static bool IsPlane()
    {
        return false;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(control_waypoint_target_road_type);
        fun(control_waypoint_id);
        fun(control_waypoint_lane_id_list);
        fun(control_waypoint_raw_lane_id_list);
        fun(control_waypoint_offset);
        fun(control_wp_extendlane_list);
        fun(deceleration_lane_type);
        fun(control_waypoint_speed_limit);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(control_waypoint_target_road_type);
        fun(control_waypoint_id);
        fun(control_waypoint_lane_id_list);
        fun(control_waypoint_raw_lane_id_list);
        fun(control_waypoint_offset);
        fun(control_wp_extendlane_list);
        fun(deceleration_lane_type);
        fun(control_waypoint_speed_limit);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("control_waypoint_target_road_type", control_waypoint_target_road_type);
        fun("control_waypoint_id", control_waypoint_id);
        fun("control_waypoint_lane_id_list", control_waypoint_lane_id_list);
        fun("control_waypoint_raw_lane_id_list", control_waypoint_raw_lane_id_list);
        fun("control_waypoint_offset", control_waypoint_offset);
        fun("control_wp_extendlane_list", control_wp_extendlane_list);
        fun("deceleration_lane_type", deceleration_lane_type);
        fun("control_waypoint_speed_limit", control_waypoint_speed_limit);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("control_waypoint_target_road_type", control_waypoint_target_road_type);
        fun("control_waypoint_id", control_waypoint_id);
        fun("control_waypoint_lane_id_list", control_waypoint_lane_id_list);
        fun("control_waypoint_raw_lane_id_list", control_waypoint_raw_lane_id_list);
        fun("control_waypoint_offset", control_waypoint_offset);
        fun("control_wp_extendlane_list", control_wp_extendlane_list);
        fun("deceleration_lane_type", deceleration_lane_type);
        fun("control_waypoint_speed_limit", control_waypoint_speed_limit);
    }

    friend bool operator==(const ::dpc::ControlWayPointStruct& lhs, const ::dpc::ControlWayPointStruct& rhs) noexcept
    {
        return (lhs.control_waypoint_target_road_type == rhs.control_waypoint_target_road_type) && (lhs.control_waypoint_id == rhs.control_waypoint_id) && (lhs.control_waypoint_lane_id_list == rhs.control_waypoint_lane_id_list) && (lhs.control_waypoint_raw_lane_id_list == rhs.control_waypoint_raw_lane_id_list) && (fabs(static_cast<double>(lhs.control_waypoint_offset - rhs.control_waypoint_offset)) < DBL_EPSILON) && (lhs.control_wp_extendlane_list == rhs.control_wp_extendlane_list) && (lhs.deceleration_lane_type == rhs.deceleration_lane_type) && (lhs.control_waypoint_speed_limit == rhs.control_waypoint_speed_limit);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_CONTROLWAYPOINTSTRUCT_H
